unit ReportInteractInfo;

interface

uses
  Classes, ScFileArchiver, XmlDoc, XmlIntf;

Type
  TInteractInfo = class
  private
    FTemplateFile: string;
    FInteractFlag: Integer;
    FInteractSubFlag: Integer;

    FMemoryStream: TMemoryStream;
    FAudits: TStringList;
    FSql: string;

    procedure LoadReportTemplate;

    procedure LoadAuditor(AXmlNode: IXMLNode);
    procedure LoadSql(AXmlNode: IXMLNode);
    procedure LoadInteractInfo(AXmlNode: IXMLNode);
    procedure LoadXmlFromStream;
  public
    constructor Create(const AFileName: string);
    destructor Destroy; override;

    property Audits: TStringList read FAudits;
    property Sql: string read FSql;
  end;

implementation

uses
  UtilMethods;

{ TInteractInfo }

constructor TInteractInfo.Create(const AFileName: string);
begin
  FTemplateFile := AFileName;
  FAudits := TStringList.Create;
  LoadReportTemplate;
  LoadXmlFromStream;
end;

destructor TInteractInfo.Destroy;
begin
  FMemoryStream.Free;
  inherited;
end;

procedure TInteractInfo.LoadAuditor(AXmlNode: IXMLNode);
var
  vAudits: IXMLNodeList;
  vAudit: IXMLNode;
  i: Integer;
begin
  if AXmlNode = nil then Exit;

  vAudits := AXmlNode.ChildNodes;
  for i := 0 to vAudits.Count - 1 do
  begin
    vAudit := AXmlNode.ChildNodes.Nodes[i];
    if vAudit.HasAttribute('ְλ') then
      FAudits.Add(vAudit.Attributes['ְλ']);
  end;
end;

procedure TInteractInfo.LoadInteractInfo(AXmlNode: IXMLNode);
var
  XmlNode: IXMLNode;
begin
  XmlNode := AXmlNode.ChildNodes.FindNode('Ϣ');
  LoadAuditor(XmlNode.ChildNodes.FindNode(''));
  LoadSql(XmlNode.ChildNodes.FindNode('ѯ'));
end;

procedure TInteractInfo.LoadReportTemplate;
var
  vArchiver: TReportArchiver;
begin
  vArchiver := TReportArchiver.Create;
  vArchiver.FileName := FTemplateFile;
  FMemoryStream := vArchiver.Extract;
end;

procedure TInteractInfo.LoadSql(AXmlNode: IXMLNode);
begin
  if AXmlNode = nil then Exit;
  if AXmlNode.HasAttribute('Sql') then
    FSql := AXmlNode.Attributes['Sql'];
end;

procedure TInteractInfo.LoadXmlFromStream;
var
  FXmlDocument: IXMLDocument;
begin
  FXmlDocument := TXMLDocument.Create(nil) as IXMLDocument;
  try
    FXmlDocument.Options := [doNodeAutoCreate,doNodeAutoIndent,doAutoPrefix,doNamespaceDecl];
    if not Assigned(FMemoryStream) then Exit;
    FXmlDocument.LoadFromStream(FMemoryStream);
    LoadInteractInfo(FXmlDocument.DocumentElement);
  finally
    FXmlDocument := nil;
  end;
end;

end.
