unit ReportManager;

interface

uses
  rmGclBillsBGDm, rmGclBillsCompareDm, rmGridHeaderDm, rmFxBillsCompareDm,
  rmBGLExecutionDm, rmGclBillsAddDm, rmFxBillsAddDm, rmBillsGatherDm,
  rmGclBillsAuditCompareDm, rmDealInfosDm, rmXmjBGLDetailDm,
  rmOtherReport1Dm, rmGcl_XmjBillsDm, rmGclBillsPlaneDm,
  rmMentalCustomized1Dm, rmCustomized2Dm, rmHaBaiCustomizedDm,
  rmTestFrm,
  ReportInteractInfo,
  Classes, ScFileArchiver, IniFiles, Graphics, DB, ProjectData,
  sdDB, ADODB;

type
  TTemplateNode = class
  private
    FFileName: string;

    FPropertyStr: String;

    FTemplateName: string;
    FClassNum: string;
    FClassName: string;
    FSubClassNum: string;
    FSubClassName: string;
    FHasPriceChange: Boolean;
    FIsMulti: Boolean;
    FIsExtra: Boolean;
    FInteractFlag: Integer;
    FInteractSubFlag: Integer;

    FInteractInfo: TInteractInfo;

    procedure ReadPropertyStr;
    procedure AnalyzePropertyStr;
    procedure LoadReportInteract;

    procedure LoadTemplateProperties;
    function GetMultiFileNames(APhaseIndex: Integer): string;
  public
    constructor Create(AFileName: string);
    destructor Destroy; override;

    property FileName: string read FFileName write FFileName;

    property TemplateName: string read FTemplateName;
    property ClassNum: string read FClassNum;
    property ClassName: string read FClassName;
    property SubClassNum: string read FSubClassNum;
    property SubClassName: string read FSubClassName;
    property HasPriceChange: Boolean read FHasPriceChange;
    // ɸѡ
    property IsMulti: Boolean read FIsMulti;
    property MultiFileNames[APhaseIndex: Integer]: string read GetMultiFileNames;
    // 
    property IsExtra: Boolean read FIsExtra;
    //  -- Ҫûѡָݱ
    property InteractFlag: Integer read FInteractFlag;
    property InteractSubFlag: Integer read FInteractSubFlag;

    property InteractInfo: TInteractInfo read FInteractInfo;
  end;

  PTemplateNode = ^TTemplateNode;

  TTemplateManager = class
  private
    FTemplateList: TList;
    FTemplateNode: TTemplateNode;

    procedure Clear;
    function GetCount: Integer;
    function GetTemplate(AIndex: Integer): TTemplateNode;
  public
    constructor Create;
    destructor Destroy; override;

    procedure LoadReportTemplates(const sTemplateDir: String);

    function AddReportTemplate(const AFileName: string): TTemplateNode;
    procedure DeleteReportTemplate(ANode: TTemplateNode);

    property Count: Integer read GetCount;
    property Template[AIndex: Integer]: TTemplateNode read GetTemplate;

    property Current: TTemplateNode read FTemplateNode write FTemplateNode;
  end;

  TReportConfig = class
  private
    FIniFile: TIniFile;

    FLeftEdge: Integer;
    FRightEdge: Integer;
    FDownEdge: Integer;
    FUpEdge: Integer;

    FContentIsNarrow: Boolean;
    FRepCellVerLine: Boolean;
    FRepCellHorLine: Boolean;
    FRepBorderUnderLine: Boolean;
    FAutoRetLine: Boolean;
    FRepBorderVerLine: Boolean;
    FBorderLine: Integer;
    FReportCellLine: Double;
    FPageSize: string;
    FGridHeaderFont: TFont;
    FHeaderFont: TFont;
    FTitleFont: TFont;
    FColumnFont: TFont;
    FGatherFont: TFont;
    FContentFont: TFont;
    FGatherLevel: Integer;
    FGclCompare: Integer;
    FXmjCompare: Integer;

    procedure LoadPaperSettings;
    procedure LoadPageSettings;
    procedure LoadGatherSettings;

    procedure SavePaperSettings;
    procedure SavePageSettings;
    procedure SaveGatherSettings;

    function GetBorderUnderLineWidth: Integer;
    function GetBorderVerLineWidth: Integer;
    function GetCellHorLineWidth: Double;
    function GetCellVerLineWidth: Double;
  public
    constructor Create;
    destructor Destroy; override;

    procedure Load;
    procedure Save;

    //------------- ֽ -------------
    // Size
    property PageSize: string read FPageSize write FPageSize;
    // ϱ߾
    property UpEdge: Integer read FUpEdge write FUpEdge;
    // ±߾
    property DownEdge: Integer read FDownEdge write FDownEdge;
    // ߾
    property LeftEdge: Integer read FLeftEdge write FLeftEdge;
    // ұ߾
    property RightEdge: Integer read FRightEdge write FRightEdge;

    //------------- ҳ -------------
    {}
    // 
    property TitleFont: TFont read FTitleFont write FTitleFont;
    // б
    property ColumnFont: TFont read FColumnFont write FColumnFont;
    // 
    property ContentFont: TFont read FContentFont write FContentFont;
    // ϼ
    property GatherFont: TFont read FGatherFont write FGatherFont;
    // ü/
    property GridHeaderFont: TFont read FGridHeaderFont write FGridHeaderFont;
    // ҳü/ҳ
    property HeaderFont: TFont read FHeaderFont write FHeaderFont;
    {}
    // ߴ
    property ReportCellLine: Double read FReportCellLine write FReportCellLine;
    // ߿ߴ
    property BorderLine: Integer read FBorderLine write FBorderLine;
    // ߿
    property RepBorderUnderLine: Boolean read FRepBorderUnderLine write FRepBorderUnderLine;
    property BorderUnderLineWidth: Integer read GetBorderUnderLineWidth;
    // ߿
    property RepBorderVerLine: Boolean read FRepBorderVerLine write FRepBorderVerLine;
    property BorderVerLineWidth: Integer read GetBorderVerLineWidth;
    // 
    property RepCellHorLine: Boolean read FRepCellHorLine write FRepCellHorLine;
    property CellHorLineWidth: Double read GetCellHorLineWidth;
    // 
    property RepCellVerLine: Boolean read FRepCellVerLine write FRepCellVerLine;
    property CellVerLineWidth: Double read GetCellVerLineWidth;
    // Զдӡ
    property AutoRetLine: Boolean read FAutoRetLine write FAutoRetLine;
    // խ
    property ContentIsNarrow: Boolean read FContentIsNarrow write FContentIsNarrow;

    //-------------  -------------
    property GatherLevel: Integer read FGatherLevel write FGatherLevel;
    property XmjCompare: Integer read FXmjCompare write FXmjCompare;
    property GclCompare: Integer read FGclCompare write FGclCompare;
  end;

  TMemoryReportManager = class
  private
    FrmGridHeaderData: TrmGridHeaderData;
    FrmGclBillsBGData: TrmGclBillsBGData;
    FrmGclBillsCompareData: TrmGclBillsCompareData;
    FrmFxBillsCompareData: TrmFxBillsCompareData;
    FrmBGLExecutionData: TrmBGLExecutionData;
    FrmGclBillsAddData: TrmGclBillsAddData;
    FrmFxBillsAddData: TrmFxBillsAddData;
    FrmBillsGatherData: TrmBillsGatherData;
    FrmGclBillsAuditCompareData: TrmGclBillsAuditCompareData;
    FrmDealInfosData: TrmDealInfosData;
    FrmXmjBGLDetailData: TrmXmjBGLDetailData;
    FrmOtherReport1Data: TrmOtherReport1Data;
    FrmGcl_XmjBillsData: TrmGcl_XmjBillsData;
    FrmGclBillsPlaneData: TrmGclBillsPlaneData;
    FrmMentalCustomized1Data: TrmMentalCustomized1Data;
    FrmCustomized2Data: TrmCustomized2Data;
    FrmHaBaiCustomizedData: TrmHaBaiCustomizedData;

    FMemoryQuery: TADOQuery;

    FProjectID: Integer;
    FTenders: TList;          // ѡܱκ޳ĿڵݣΪһת
    FProjects: TList;         // ͨ
    FE_PCDProjects: TList;    // ѡƣʩͼ
    FE_AProjects: TList;      // ѡ׼
    FDealProjects: TList;     // ѡͬĿ

    function OpenProject(ARec: TsdDataRecord): TProjectData;
    procedure FreeProject(AProjectData: TProjectData);

    function SelectProjectID: Integer;
    function SelectProjectIDs(AProjects: TList; ADataSetID: Integer): Boolean;
    procedure GenerateTenders(AProjects: TList);

    function SelectSourceAndAssignData(ADataSetID: Integer;
      AProjectData: TProjectData): TDataSet;
    function SelectProjectsAndAssignData(ADataSetID: Integer): TDataSet;
  public
    constructor Create;
    destructor Destroy; override;

    function GetMemoryDataSet(ADataSetID: Integer;
      AProjectData: TProjectData): TDataSet;
    function GetSqlResultDataSet(const ASql: string; AProjectData: TProjectData): TDataSet;
  end;

implementation

uses
  SysUtils, ZhAPI, UtilMethods, Globals, ProjectManagerDm,
  rmSelectProjectFrm, Controls;

{ TTemplateNode }

procedure TTemplateNode.AnalyzePropertyStr;
var
  sgs: TStrings;
  I: Integer;
begin
  if FPropertyStr <> '' then
  begin
    sgs := TStringList.Create;
    try
      sgs.Text := FPropertyStr;
      for I := 0 to sgs.Count - 1 do
      begin
        if SameText(sgs.Names[I], '') then
          FTemplateName := sgs.Values[sgs.Names[I]]
        else if SameText(sgs.Names[I], '') then
          FClassNum := sgs.Values[sgs.Names[I]]
        else if SameText(sgs.Names[I], '') then
          FClassName := sgs.Values[sgs.Names[I]]
        else if SameText(sgs.Names[I], 'ӷ') then
          FSubClassNum := sgs.Values[sgs.Names[I]]
        else if SameText(sgs.Names[I], 'ӷ') then
          FSubClassName := sgs.Values[sgs.Names[I]]
        else if SameText(sgs.Names[I], '۱') then
          FHasPriceChange := SameText(sgs.Values[sgs.Names[I]], '')
        else if SameText(sgs.Names[I], 'ɸѡ') then
          FIsMulti := SameText(sgs.Values[sgs.Names[I]], '')
        else if SameText(sgs.Names[I], '') then
          FIsExtra := SameText(sgs.Values[sgs.Names[I]], '')
        else if SameText(sgs.Names[I], '') then
          FInteractFlag := StrToIntDef(sgs.Values[sgs.Names[I]], 0)
        else if SameText(sgs.Names[I], '') then
          FInteractSubFlag := StrToIntDef(sgs.Values[sgs.Names[I]], 0);
      end;
    finally
      sgs.Free;
    end;
  end;
end;

constructor TTemplateNode.Create(AFileName: string);
begin
  FFileName := AFileName;
  FInteractFlag := 0;
  FInteractSubFlag := 0;
  LoadTemplateProperties;
end;

destructor TTemplateNode.Destroy;
begin
  if Assigned(FInteractInfo) then
    FInteractInfo.Free;
  inherited;
end;

function TTemplateNode.GetMultiFileNames(APhaseIndex: Integer): string;
var
  sName, sPath: string;
begin
  sPath := ExtractFilePath(FFileName);
  sName := ExtractSimpleFileName(FFileName);
  Result := Format('%s%s\%s_%d.srt', [sPath, sName, sName, APhaseIndex]);
end;

procedure TTemplateNode.LoadReportInteract;
begin
  FInteractInfo := TInteractInfo.Create(FileName);
end;

procedure TTemplateNode.LoadTemplateProperties;
begin
  try
    ReadPropertyStr;
    AnalyzePropertyStr;
    if FIsExtra then
      LoadReportInteract;
  except
    on E: Exception do
    begin
      TipMessage(Format('ļ[%s]޷򿪣Ϣ'#13#10'%s.'#13#10 +
        'ݺͷϵ(0756)3850888', [FFileName, E.Message]));
    end;
  end;
end;

procedure TTemplateNode.ReadPropertyStr;
var
  vStream: TFileStream;
  FileHead: TScFileHead;
begin
  vStream := TFileStream.Create(FFileName, fmOpenRead);
  try
    vStream.Seek($00, soFromBeginning);
    vStream.Read(FileHead, Sizeof(TScFileHead));
    if FileHead.ReportProperties[0] <> #0 then
      SetString(FPropertyStr, FileHead.ReportProperties, 256);
  finally
    vStream.Free;
  end;
end;

{ TTemplateManager }

function TTemplateManager.AddReportTemplate(const AFileName: string): TTemplateNode;
begin
  Result := TTemplateNode.Create(AFileName);
  FTemplateList.Add(Result);
end;

procedure TTemplateManager.Clear;
begin
  ClearObjects(FTemplateList);
end;

constructor TTemplateManager.Create;
begin
  FTemplateList := TList.Create;
  LoadReportTemplates(GetAppFilePath + 'ReportTemplates');
end;

procedure TTemplateManager.DeleteReportTemplate(ANode: TTemplateNode);
begin
  FTemplateList.Remove(ANode);
  ANode.Free;
end;

destructor TTemplateManager.Destroy;
begin
  Clear;
  FTemplateList.Free;
  inherited;
end;

function TTemplateManager.GetCount: Integer;
begin
  Result := FTemplateList.Count;
end;

function TTemplateManager.GetTemplate(AIndex: Integer): TTemplateNode;
begin
  Result := TTemplateNode(FTemplateList.Items[AIndex]);
end;

procedure TTemplateManager.LoadReportTemplates(const sTemplateDir: String);
var
  sFiles: TStringList;
  I: Integer;
begin
  Clear;
  sFiles := TStringList.Create;
  try
    FilterFiles(sTemplateDir, '.srt', sFiles);
    for I := 0 to sFiles.Count - 1 do
      AddReportTemplate(sTemplateDir + '\' + sFiles[I]);
  finally
    sFiles.Free;
  end;
end;

{ TReportConfig }

constructor TReportConfig.Create;
begin
  inherited Create;
  FTitleFont := TFont.Create;
  FColumnFont := TFont.Create;
  FContentFont := TFont.Create;
  FGatherFont := TFont.Create;
  FGridHeaderFont := TFont.Create;
  FHeaderFont := TFont.Create;
  Load;
end;

destructor TReportConfig.Destroy;
begin
  Save;
  FHeaderFont.Free;
  FGridHeaderFont.Free;
  FGatherFont.Free;
  FContentFont.Free;
  FColumnFont.Free;
  FTitleFont.Free;
  inherited;
end;

function TReportConfig.GetBorderUnderLineWidth: Integer;
begin
  Result := 0;
  if RepBorderUnderLine then
    Result := BorderLine;
end;

function TReportConfig.GetBorderVerLineWidth: Integer;
begin
  Result := 0;
  if RepBorderVerLine then
    Result := BorderLine;
end;

function TReportConfig.GetCellHorLineWidth: Double;
begin
  Result := 0;
  if RepCellHorLine then
    Result := ReportCellLine;
end;

function TReportConfig.GetCellVerLineWidth: Double;
begin
  Result := 0;
  if RepCellVerLine then
    Result := ReportCellLine;
end;

procedure TReportConfig.Load;
begin
  try
    FIniFile := TIniFile.Create(GetAppFilePath + 'Report.ini');
    LoadPaperSettings;
    LoadPageSettings;
    LoadGatherSettings;
  finally
    FIniFile.Free;
  end;
end;

procedure TReportConfig.LoadGatherSettings;
begin
  FGatherLevel := FIniFile.ReadInteger('', '', 5);
  FXmjCompare := FIniFile.ReadInteger('', 'Ŀڹ', 0);
  FGclCompare := FIniFile.ReadInteger('', '嵥', 0);
end;

procedure TReportConfig.LoadPageSettings;

  procedure LoadFont(const AFontName: string; AFont: TFont);
  begin
    AFont.Name := FIniFile.ReadString('ҳ', AFontName + '_', 'SmartSumSun');
    AFont.Size := FIniFile.ReadInteger('ҳ', AFontName + '_С', 9);
    if FIniFile.ReadBool('ҳ', AFontName + '_', False) then
      AFont.Style := AFont.Style + [fsBold]
    else
     AFont.Style := AFont.Style - [fsBold];
    if FIniFile.ReadBool('ҳ', AFontName + '_б', False) then
      AFont.Style := AFont.Style + [fsItalic]
    else
     AFont.Style := AFont.Style - [fsItalic];
    if FIniFile.ReadBool('ҳ', AFontName + '_»', False) then
      AFont.Style := AFont.Style + [fsUnderline]
    else
      AFont.Style := AFont.Style - [fsUnderline];
    //AFont.Height := FIniFile.ReadInteger('ҳ', AFontName + '_и', 9);
  end;

begin
  LoadFont('', TitleFont);
  LoadFont('б', ColumnFont);
  LoadFont('', ContentFont);
  LoadFont('ϼ', GatherFont);
  LoadFont('ü/', GridHeaderFont);
  LoadFont('ҳü/ҳ', HeaderFont);
  FReportCellLine := FIniFile.ReadFloat('ҳ', 'ߴ', 0.75);
  FBorderLine := FIniFile.ReadInteger('ҳ', '߿ߴ', 2);
  FRepBorderUnderLine := FIniFile.ReadBool('ҳ', '߿', True);
  FRepBorderVerLine := FIniFile.ReadBool('ҳ', '߿', False);
  FRepCellHorLine := FIniFile.ReadBool('ҳ', '', True);
  FRepCellVerLine := FIniFile.ReadBool('ҳ', '', True);
  FAutoRetLine := FIniFile.ReadBool('ҳ', 'Զдӡ', False);
  FContentIsNarrow := FIniFile.ReadBool('ҳ', 'խ', False);
end;

procedure TReportConfig.LoadPaperSettings;
begin
  FPageSize := FIniFile.ReadString('ֽ', '', 'A4');
  FUpEdge := FIniFile.ReadInteger('ֽ', 'ϱ߾', 12);
  FDownEdge := FIniFile.ReadInteger('ֽ', '±߾', 12);
  FLeftEdge := FIniFile.ReadInteger('ֽ', '߾', 12);
  FRightEdge := FIniFile.ReadInteger('ֽ', 'ұ߾', 12);
end;

procedure TReportConfig.Save;
begin
  try
    FIniFile := TIniFile.Create(GetAppFilePath + 'Report.ini');
    SavePaperSettings;
    SavePageSettings;
  finally
    FIniFile.Free;
  end;
end;

procedure TReportConfig.SaveGatherSettings;
begin
  FIniFile.WriteInteger('', '', FGatherLevel);
  FIniFile.WriteInteger('', 'Ŀڹ', FXmjCompare);
  FIniFile.WriteInteger('', '嵥', FGclCompare);
end;

procedure TReportConfig.SavePageSettings;

  procedure SaveFont(const AFontName: string; AFont: TFont);
  begin
    FIniFile.WriteString('ҳ', AFontName + '_', AFont.Name);
    FIniFile.WriteInteger('ҳ', AFontName + '_С', AFont.Size);
    FIniFile.WriteBool('ҳ', AFontName + '_', fsBold in AFont.Style);
    FIniFile.WriteBool('ҳ', AFontName + '_б', fsItalic in AFont.Style);
    FIniFile.WriteBool('ҳ', AFontName + '_»', fsUnderline in AFont.Style);
    FIniFile.WriteInteger('ҳ', AFontName + '_и', AFont.Height);
  end;

begin
  SaveFont('', TitleFont);
  SaveFont('б', ColumnFont);
  SaveFont('', ContentFont);
  SaveFont('ϼ', GatherFont);
  SaveFont('ü/', GridHeaderFont);
  SaveFont('ҳü/ҳ', HeaderFont);
  FIniFile.WriteFloat('ҳ', 'ߴ', FReportCellLine);
  FIniFile.WriteInteger('ҳ', '߿ߴ', FBorderLine);
  FIniFile.WriteBool('ҳ', '߿', FRepBorderUnderLine);
  FIniFile.WriteBool('ҳ', '߿', FRepBorderVerLine);
  FIniFile.WriteBool('ҳ', '', FRepCellHorLine);
  FIniFile.WriteBool('ҳ', '', FRepCellVerLine);
  FIniFile.WriteBool('ҳ', 'Զдӡ', FAutoRetLine);
  FIniFile.WriteBool('ҳ', 'խ', FContentIsNarrow);
end;

procedure TReportConfig.SavePaperSettings;
begin
  FIniFile.WriteString('ֽ', '', FPageSize);
  FIniFile.WriteInteger('ֽ', 'ϱ߾', FUpEdge);
  FIniFile.WriteInteger('ֽ', '±߾', FDownEdge);
  FIniFile.WriteInteger('ֽ', '߾', FLeftEdge);
  FIniFile.WriteInteger('ֽ', 'ұ߾', FRightEdge);
end;

{ TMemoryReportManager }

constructor TMemoryReportManager.Create;
begin
  FrmGridHeaderData := TrmGridHeaderData.Create(nil);
  FrmGclBillsBGData := TrmGclBillsBGData.Create;
  FrmGclBillsCompareData := TrmGclBillsCompareData.Create;
  FrmFxBillsCompareData :=  TrmFxBillsCompareData.Create(nil);
  FrmBGLExecutionData := TrmBGLExecutionData.Create(nil);
  FrmGclBillsAddData := TrmGclBillsAddData.Create(nil);
  FrmFxBillsAddData := TrmFxBillsAddData.Create(nil);
  FrmBillsGatherData := TrmBillsGatherData.Create(nil);
  FrmGclBillsAuditCompareData := TrmGclBillsAuditCompareData.Create;
  FrmDealInfosData := TrmDealInfosData.Create(nil);
  FrmXmjBGLDetailData := TrmXmjBGLDetailData.Create(nil);
  FrmOtherReport1Data := TrmOtherReport1Data.Create(nil);
  FrmGcl_XmjBillsData := TrmGcl_XmjBillsData.Create(nil);
  FrmGclBillsPlaneData := TrmGclBillsPlaneData.Create(nil);
  FrmMentalCustomized1Data := TrmMentalCustomized1Data.Create(nil);
  FrmCustomized2Data := TrmCustomized2Data.Create(nil);
  FrmHaBaiCustomizedData := TrmHaBaiCustomizedData.Create(nil);

  FMemoryQuery := TADOQuery.Create(nil);

  FProjectID := -1;
  FTenders :=  TList.Create;
  FProjects := TList.Create;
  FE_PCDProjects := TList.Create;
  FE_AProjects := TList.Create;
  FDealProjects := TList.Create;
end;

destructor TMemoryReportManager.Destroy;
begin
  FTenders.Free;
  ClearObjects(FProjects);
  FProjects.Free;
  ClearObjects(FE_PCDProjects);
  FE_PCDProjects.Free;
  ClearObjects(FE_AProjects);
  FE_AProjects.Free;
  ClearObjects(FDealProjects);
  FDealProjects.Free;

  FMemoryQuery.Free;

  FrmHaBaiCustomizedData.Free;
  FrmCustomized2Data.Free;
  FrmMentalCustomized1Data.Free;
  FrmGclBillsPlaneData.Free;
  FrmGcl_XmjBillsData.Free;
  FrmOtherReport1Data.Free;
  FrmXmjBGLDetailData.Free;
  FrmDealInfosData.Free;
  FrmGclBillsAuditCompareData.Free;
  FrmBillsGatherData.Free;
  FrmFxBillsAddData.Free;
  FrmGclBillsAddData.Free;
  FrmBGLExecutionData.Free;
  FrmFxBillsCompareData.Free;
  FrmGclBillsCompareData.Free;
  FrmGclBillsBGData.Free;
  FrmGridHeaderData.Free;
  inherited;
end;

procedure TMemoryReportManager.FreeProject(AProjectData: TProjectData);
begin
  if not Assigned(OpenProjectManager.FindProjectData(AProjectData.ProjectID)) then
    AProjectData.Free;
end;

procedure TMemoryReportManager.GenerateTenders(AProjects: TList);
var
  i: Integer;
  SelectProject: TSelectProject;
begin
  FTenders.Clear;
  for i := 0 to AProjects.Count - 1 do
  begin
    SelectProject := TSelectProject(AProjects.Items[i]);
    if SelectProject.IsTender then
      FTenders.Add(SelectProject);
  end;
end;

function TMemoryReportManager.GetMemoryDataSet(
  ADataSetID: Integer; AProjectData: TProjectData): TDataSet;
begin
  {if FrmHaBaiCustomizedData.hbGatherType = hbgt_CS then
    ADataSetID := 51
  else
    ADataSetID := 50;}
  case ADataSetID of
    1: Result := FrmGridHeaderData.AssginData(AProjectData);   // ͷ
    2: Result := FrmGclBillsBGData.AssignData(AProjectData, rmtPhaseGather, rmgtGather); // 嵥[]
    3, 4, 41:
    // 0102, ̨ 07
       Result := SelectSourceAndAssignData(ADataSetID, AProjectData);
    5: Result := FrmBGLExecutionData.AssignData(AProjectData); // ֧09
    6: Result := FrmGclBillsBGData.AssignData(AProjectData, rmtEndGather, rmgtGather); // 嵥[ֹ]
    7: Result := FrmGclBillsAddData.AssignData(AProjectData); // 02
    8: Result := FrmFxBillsAddData.AssignData(AProjectData); // 02-1
    9, 10, 11, 12, 13,
    // 04, 05, 05-2, 04-1, 02,
       20, 21, 22, 23, 24, 25, 26, 27, 28, 29,
       // 02/03, 07, 08, 09, 11, 12, 16, 17, 18, 19,
       32, 33, 35, 36, 39, 40, 44,
       // 9, 06-1, 04, 01, 嵥[߹ֶܾ], [㶫춨]ܱ
       45, 46, 47, 48, 49, 50,
       // ɹŹ׶Ʊ(֧2, ֧3, ֧14, ֧3-1, ֧4-1, ǩ)
       52, 53, 54, 55:
       // ɹ306(֧2, ֧3, ֧3-1, ֧4-1)
        Result := SelectProjectsAndAssignData(ADataSetID);
    14: Result := FrmGclBillsBGData.AssignData(AProjectData, rmtPhaseDeal, rmgtGather); // 嵥[ںͬ]
    15: Result := FrmGclBillsBGData.AssignData(AProjectData, rmtPhaseQc, rmgtGather); // 嵥[]
    16: Result := FrmGclBillsBGData.AssignData(AProjectData, rmtPhasePc, rmgtGather); // 嵥[ڵ۱]
    17: Result := FrmGclBillsBGData.AssignData(AProjectData, rmtEndDeal, rmgtGather); // 嵥[ֹںͬ]
    18: Result := FrmGclBillsBGData.AssignData(AProjectData, rmtEndQc, rmgtGather); // 嵥[ֹ]
    19: Result := FrmGclBillsBGData.AssignData(AProjectData, rmtEndPc, rmgtGather); // 嵥[ֹڵ۱]
    30: Result := FrmGclBillsBGData.AssignData(AProjectData, rmtPhaseQc, rmgtFlow);
    31: Result := FrmGclBillsAuditCompareData.AssignData(AProjectData, rmftPhaseGather); // 嵥[ԭ&]
    34: Result := FrmXmjBGLDetailData.AssignData(AProjectData); // ̨
    37: Result := FrmGcl_XmjBillsData.AssignData(AProjectData, gxtTopGcl); // 嵥
    38: Result := FrmGclBillsPlaneData.AssignData(AProjectData); // 嵥ƽ[0̨]
    42: Result := FrmGcl_XmjBillsData.AssignData(AProjectData, gxtFlowGcl); // ϱ׼̨ϸ
    43: Result := FrmGcl_XmjBillsData.AssignData(AProjectData, gxtWithoutXmj); // ϱ׼嵥ܱ
    51: Result := FrmHaBaiCustomizedData.AssignAssistantData(hbgt_CS); //ɹŹס306Ʊ--ǩ
  end;
  // For Inner Test
  //ShowTestForm(Result);
end;

function TMemoryReportManager.GetSqlResultDataSet(const ASql: string;
  AProjectData: TProjectData): TDataSet;
begin
  if ASql <> '' then
  begin
    FMemoryQuery.Close;
    FMemoryQuery.Connection := AProjectData.ADOConnection;
    FMemoryQuery.SQL.Clear;
    FMemoryQuery.SQL.Add(ASql);
    FMemoryQuery.Open;
  end;
  Result := FMemoryQuery;
  //ShowTestForm(Result);
end;

function TMemoryReportManager.OpenProject(
  ARec: TsdDataRecord): TProjectData;
begin
  Result := OpenProjectManager.FindProjectData(ARec.ValueByName('ID').AsInteger);
  if not Assigned(Result) then
  begin
    Result := TProjectData.Create;
    Result.OpenForReport(GetMyProjectsFilePath + ARec.ValueByName('FileName').AsString);
  end;
end;

function TMemoryReportManager.SelectProjectID: Integer;
var
  SelectFrm: TProjectSelectForm;
begin
  Result := -1;
  SelectFrm := TProjectSelectForm.Create(False, FProjectID, FProjects);
  //SelectFrm.LoadHistorySelect(FProjectID);
  try
    if SelectFrm.ShowModal = mrOk then
    begin
      SelectFrm.SelectResult(FProjectID);
      Result := FProjectID;
    end;
  finally
    SelectFrm.Free;
  end;
end;

function TMemoryReportManager.SelectProjectIDs(AProjects: TList;
  ADataSetID: Integer): Boolean;
var
  SelectFrm: TProjectSelectForm;
begin
  Result := False;
  SelectFrm := TProjectSelectForm.Create(True, FProjectID, AProjects);
  case ADataSetID of
    13, 33: SelectFrm.MultiSelectType := mstE_PCD;
    20, 36: SelectFrm.MultiSelectType := mstE_A;
    32: SelectFrm.MultiSelectType := mstDeal;
    40: SelectFrm.MultiSelectType := mstMental1;
    else SelectFrm.MultiSelectType := mstCommon;
  end;

  try
    if SelectFrm.ShowModal = mrOk then
    begin
      SelectFrm.SelectResult(AProjects);
      Result := True;
    end;
  finally
    SelectFrm.Free;
  end;
end;

function TMemoryReportManager.SelectProjectsAndAssignData(
  ADataSetID: Integer): TDataSet;
var
  Projects: TList;
begin
  Result := nil;
  case ADataSetID of
    13, 33: Projects := FE_PCDProjects;
    20, 36: Projects := FE_AProjects;
    32: Projects := FDealProjects;
    else Projects := FProjects;
  end;

  if SelectProjectIDs(Projects, ADataSetID) then
  begin
    case ADataSetID of
      9: FrmBillsGatherData.GatherType := bgtFx;
      10: FrmBillsGatherData.GatherType := bgtXmj;
      11: FrmBillsGatherData.GatherType := bgtGcl;
      12: FrmBillsGatherData.GatherType := bgtMultiXmj;
      13: FrmBillsGatherData.GatherType := bgtEstimate;  // 02
      20: FrmBillsGatherData.GatherType := bgtEstimate1; // 02/03
      21: FrmBillsGatherData.GatherType := bgtFinal07; // 07
      22: FrmBillsGatherData.GatherType := bgtFinal08;
      23: FrmBillsGatherData.GatherType := bgtFinal09;
      24: FrmBillsGatherData.GatherType := bgtFinal11;
      25: FrmBillsGatherData.GatherType := bgtFinal12;
      26: FrmBillsGatherData.GatherType := bgtFinal16;
      27: FrmBillsGatherData.GatherType := bgtFinal17;
      28: FrmBillsGatherData.GatherType := bgtFinal18;
      29: FrmBillsGatherData.GatherType := bgtFinal19;
      33: FrmBillsGatherData.GatherType := bgtG_Final06_1;
      35: FrmBillsGatherData.GatherType := bgtG_Final04;
      39: FrmMentalCustomized1Data.MentalType := 0;
      40: FrmMentalCustomized1Data.MentalType := 1;
    end;
    GenerateTenders(Projects);
    case ADataSetID of
      32: Result := FrmDealInfosData.AssignData(FTenders);
      36: Result := FrmOtherReport1Data.AssignData(FTenders);
      39, 40: Result := FrmMentalCustomized1Data.AssignData(FTenders);
      44: Result := FrmCustomized2Data.AssignData(FTenders);
      45: Result := FrmHaBaiCustomizedData.AssignData(FTenders, hbgt2, True);
      46: Result := FrmHaBaiCustomizedData.AssignData(FTenders, hbgt3, True);
      47: Result := FrmHaBaiCustomizedData.AssignData(FTenders, hbgt14, True);
      48: Result := FrmHaBaiCustomizedData.AssignData(FTenders, hbgt3_1, True);
      49: Result := FrmHaBaiCustomizedData.AssignData(FTenders, hbgt4_1, True);
      50: Result := FrmHaBaiCustomizedData.AssignData(FTenders, hbgt_CS, True);
      52: Result := FrmHaBaiCustomizedData.AssignData(FTenders, hbgt2, True);
      53: Result := FrmHaBaiCustomizedData.AssignData(FTenders, hbgt3, True);
      54: Result := FrmHaBaiCustomizedData.AssignData(FTenders, hbgt3_1, True);
      55: Result := FrmHaBaiCustomizedData.AssignData(FTenders, hbgt4_1, True);
      else Result := FrmBillsGatherData.AssignData(FTenders);
    end;
  end;
end;

function TMemoryReportManager.SelectSourceAndAssignData(
  ADataSetID: Integer; AProjectData: TProjectData): TDataSet;
var
  sRec, cRec: TsdDataRecord;
  sProject, cProject: TProjectData;
begin
  sRec := ProjectManager.sddProjectsInfo.FindKey('idxID', SelectProjectID);
  cRec := ProjectManager.sddProjectsInfo.FindKey('idxID', AProjectData.ProjectID);
  if Assigned(sRec) and Assigned(cRec) then
  begin
    sProject := OpenProject(sRec);
    cProject := OpenProject(cRec);
    try
      case ADataSetID of
        4: FrmFxBillsCompareData.CompareType := fctCommon;
        41: FrmFxBillsCompareData.CompareType := fctG07;
      end;
      
      case ADataSetID of
        3: Result := FrmGclBillsCompareData.AssignData(sProject, cProject);
        4: Result := FrmFxBillsCompareData.AssignData(sProject, cProject);
        41: Result := FrmFxBillsCompareData.AssignData(sProject, cProject);
      end;
    finally
      FreeProject(sProject);
      FreeProject(cProject);
    end;
  end;
end;

end.
